package com.sophiaatkinson.pronouns;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.bukkit.plugin.java.JavaPlugin;

public class PronounsPlugin extends JavaPlugin {
    private Connection connection;
    private boolean usingMySQL;
    private List<String> blockedPronouns;
    private List<String> pronounSuggestions;
    private final List<String> errorLog = new ArrayList<>();

    @Override
    public void onEnable() {
        getLogger().info("PronounsPlugin enabled!");

        // Load blocked pronouns and pronoun suggestions from config
        saveDefaultConfig();
        blockedPronouns = getConfig().getStringList("blocked-pronouns");
        pronounSuggestions = getConfig().getStringList("pronoun-suggestions");

        // Load database
        initDatabase();

        // Register commands and placeholders
        PronounsCommand command = new PronounsCommand(this);
        getCommand("pronouns").setExecutor(command);
        getCommand("pronouns").setTabCompleter(command);

        new PronounsPlaceholder(this).register();
    }

    @Override
    public void onDisable() {
        try {
            if (connection != null) connection.close();
        } catch (SQLException e) {
            errorLog.add("Database close error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    private void initDatabase() {
        usingMySQL = getConfig().getBoolean("mysql.enabled", false);

        try {
            if (usingMySQL) {
                String host = getConfig().getString("mysql.host");
                int port = getConfig().getInt("mysql.port");
                String database = getConfig().getString("mysql.database");
                String user = getConfig().getString("mysql.user");
                String password = getConfig().getString("mysql.password");

                String url = "jdbc:mysql://" + host + ":" + port + "/" + database + "?useSSL=false&autoReconnect=true";
                connection = DriverManager.getConnection(url, user, password);
            } else {
                File dbFile = new File(getDataFolder(), "pronouns.db");
                if (!getDataFolder().exists()) getDataFolder().mkdirs();
                String url = "jdbc:sqlite:" + dbFile.getAbsolutePath();
                connection = DriverManager.getConnection(url);
            }

            Statement stmt = connection.createStatement();
            stmt.executeUpdate("CREATE TABLE IF NOT EXISTS pronouns (uuid VARCHAR(36) PRIMARY KEY, pronouns TEXT);");
        } catch (SQLException e) {
            errorLog.add("Database initialization error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public void setPronouns(UUID uuid, String pronouns) {
        if (pronouns == null) {
            try (PreparedStatement ps = connection.prepareStatement("DELETE FROM pronouns WHERE uuid = ?")) {
                ps.setString(1, uuid.toString());
                ps.executeUpdate();
            } catch (SQLException e) {
                errorLog.add("Database delete error: " + e.getMessage());
                e.printStackTrace();
            }
            return;
        }
    
        for (String blocked : blockedPronouns) {
            if (pronouns.contains(blocked)) return;
        }

        try (PreparedStatement ps = connection.prepareStatement("REPLACE INTO pronouns (uuid, pronouns) VALUES (?, ?)")) {
            ps.setString(1, uuid.toString());
            ps.setString(2, pronouns);
            ps.executeUpdate();
        } catch (SQLException e) {
            // Fallback for MySQL since REPLACE INTO isn't ideal for all cases
            if (usingMySQL) {
                try (PreparedStatement ps = connection.prepareStatement("INSERT INTO pronouns (uuid, pronouns) VALUES (?, ?) ON DUPLICATE KEY UPDATE pronouns = VALUES(pronouns)")) {
                    ps.setString(1, uuid.toString());
                    ps.setString(2, pronouns);
                    ps.executeUpdate();
                    return;
                } catch (SQLException ex) {
                    errorLog.add("MySQL upsert error: " + ex.getMessage());
                    ex.printStackTrace();
                }
            }
            errorLog.add("Database update error: " + e.getMessage());
            e.printStackTrace();
        }
    }

    public String getPronouns(UUID uuid) {
        try (PreparedStatement ps = connection.prepareStatement("SELECT pronouns FROM pronouns WHERE uuid = ?")) {
            ps.setString(1, uuid.toString());
            ResultSet rs = ps.executeQuery();
            if (rs.next()) {
                return rs.getString("pronouns");
            }
        } catch (SQLException e) {
            errorLog.add("Database fetch error: " + e.getMessage());
            e.printStackTrace();
        }
        return null;
    }
    public Map<UUID, String> getAllPronouns() {
        Map<UUID, String> result = new HashMap<>();
        try (Statement stmt = connection.createStatement()) {
            ResultSet rs = stmt.executeQuery("SELECT uuid, pronouns FROM pronouns");
            while (rs.next()) {
                result.put(UUID.fromString(rs.getString("uuid")), rs.getString("pronouns"));
            }
        } catch (SQLException e) {
            errorLog.add("Database fetch all error: " + e.getMessage());
            e.printStackTrace();
        }
        return result;
    }

    public void generateDebugFile() {
        File debugFile = new File(getDataFolder(), "debug.txt");
        try {
            if (!getDataFolder().exists()) getDataFolder().mkdirs();
            if (!debugFile.exists()) debugFile.createNewFile();

            FileWriter writer = new FileWriter(debugFile);
            writer.write("=== PronounsPlugin Debug Report ===\n");
            writer.write("Plugin Enabled: " + isEnabled() + "\n");
            writer.write("Database Connected: " + (connection != null) + "\n");
            writer.write("Using MySQL: " + usingMySQL + "\n");
            writer.write("Blocked Pronouns: " + blockedPronouns + "\n");
            writer.write("Pronoun Suggestions: " + pronounSuggestions + "\n");
            writer.write("Error Log:\n");

            if (errorLog.isEmpty()) {
                writer.write("  No errors logged.\n");
            } else {
                for (String error : errorLog) {
                    writer.write("  - " + error + "\n");
                }
            }

            writer.write("Generated at: " + new java.util.Date() + "\n");
            writer.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public List<String> getPronounSuggestions() {
        return pronounSuggestions;
    }
}
